/**********************************************************************
 * VPI PLIbook_count_args_vpi Example -- PLI application using VPI
 * routines
 *
 * C code to test the VPI "PLIbook_count_args_vpi()" application.
 *
 * NOTE: THIS ROUTINE IS PROVIDED AS SHORT EXAMPLE ON HOW TO ACCESS
 * TASK/FUNCTION ARGUMENTS, AND HOW TO PERFORM ERROR CHECKING ON IF
 * THE DESIRED INFORMATION WAS OBTAINED.  THIS EXAMPLE IS NOT
 * INTENDED TO BE AN EFFICIENT PLI APPLICATION.  THE FILE
 * get_arg_handle_vpi_efficient CONTAINS A MORE EFFICIENT VERSION
 * OF THIS EXAMPLE.
 *
 * Usage:
 *   numargs = PLIbook_count_args_vpi(<any_number_and_type_of_args>);
 *     Returns the number of system task/function arguments.
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *********************************************************************/

#define PLIbookDebug 1 /* set to non-zero for verbose debug messages */

#define VPI_1995 0 /* set to non-zero for Verilog-1995 compatibility */

#include <stdlib.h>    /* ANSI C standard library */
#include <stdio.h>     /* ANSI C standard input/output library */
#include <stdarg.h>    /* ANSI C standard arguments library */
#include "vpi_user.h"  /* IEEE 1364 PLI VPI routine library  */

#if VPI_1995
#include "../vpi_1995_compat.h"  /* kludge new Verilog-2001 routines */
#endif

/* Prototypes of the utility applications */
int PLIbook_count_args_vpi();


/**********************************************************************
 * PLIbook_count_args_vpi()
 * Counts the number of system task/function arguments.  Similar to
 * tf_nump().
 *********************************************************************/
int PLIbook_count_args_vpi()
{
  vpiHandle systf_h, arg_itr, arg_h;
  int tfnum = 0;
  s_vpi_error_info err;  /* structure for error handling */

  systf_h = vpi_handle(vpiSysTfCall, NULL);
  #if PLIbookDebug /* if error, generate verbose debug message */
    if (vpi_chk_error(&err)) {
      vpi_printf("ERROR: PLIbook_count_args_vpi() could not obtain handle to systf call\n");
      vpi_printf("File %s, Line %d: %s\n",
                  err.file, err.line, err.message);
    }
  #else  /* if error, generate brief error message */
    if (systf_h == NULL)
      vpi_printf("ERROR: PLIbook_count_args_vpi() could not obtain handle to systf call\n");
  #endif

  arg_itr = vpi_iterate(vpiArgument, systf_h);
  #if PLIbookDebug  /* if error, generate verbose debug message */
    if (vpi_chk_error(&err)) {
      vpi_printf("ERROR: PLIbook_count_args_vpi() could not obtain iterator to systf args\n");
      vpi_printf("File %s, Line %d: %s\n",
                  err.file, err.line, err.message);
    }
  #else  /* if error, generate brief error message */
    if (arg_itr == NULL)
      vpi_printf("ERROR: PLIbook_count_args_vpi() could not obtain iterator to systf args\n");
  #endif

  while (arg_h = vpi_scan(arg_itr) ) {
    tfnum++;
  }

  return(tfnum);
}
/*********************************************************************/
