/**********************************************************************
 * $port_info example -- PLI application using VPI routines
 *
 * C source to print information about module ports.
 *
 * Usage: $port_info(<module_instance_name>);
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *********************************************************************/

#define VPI_1995 0 /* set to non-zero for Verilog-1995 compatibility */

#include <stdlib.h>    /* ANSI C standard library */
#include <stdio.h>     /* ANSI C standard input/output library */
#include <stdarg.h>    /* ANSI C standard arguments library */
#include "vpi_user.h"  /* IEEE 1364 PLI VPI routine library  */

#if VPI_1995
#include "../vpi_1995_compat.h"  /* kludge new Verilog-2001 routines */
#endif

/* prototypes of routines in this PLI application */
PLI_INT32  PLIbook_PortInfo_compiletf(PLI_BYTE8 *user_data);
PLI_INT32  PLIbook_PortInfo_calltf(PLI_BYTE8 *user_data);

/**********************************************************************
 * VPI Registration Data
 *********************************************************************/
void PLIbook_PortInfo_register()
{
  s_vpi_systf_data tf_data;
  tf_data.type        = vpiSysTask;
  tf_data.sysfunctype = 0;
  tf_data.tfname      = "$port_info";
  tf_data.calltf      = PLIbook_PortInfo_calltf;
  tf_data.compiletf   = PLIbook_PortInfo_compiletf;
  tf_data.sizetf      = NULL;
  tf_data.user_data   = NULL;

  vpi_register_systf(&tf_data);
}
/*********************************************************************/


/**********************************************************************
 * Compiletf application
 *********************************************************************/
PLI_INT32 PLIbook_PortInfo_compiletf(PLI_BYTE8 *user_data)
{
  vpiHandle systf_h, tfarg_itr, tfarg_h;

  systf_h = vpi_handle(vpiSysTfCall, NULL);
  if (systf_h == NULL) {
    vpi_printf("ERROR: list_pathout_ports could not obtain handle to systf call\n");
    vpi_control(vpiFinish, 1);  /* abort simulation */
    return(0);
  }
  tfarg_itr = vpi_iterate(vpiArgument, systf_h);
  if (systf_h == NULL) {
    vpi_printf("ERROR: list_pathout_ports could not obtain iterator to systf args\n");
    vpi_control(vpiFinish, 1);  /* abort simulation */
    return(0);
  }
  tfarg_h = vpi_scan(tfarg_itr);
  if (vpi_get(vpiType, tfarg_h) != vpiModule) {
    vpi_printf("ERROR: $list_pathout_ports arg must be module instance\n");
    vpi_control(vpiFinish, 1);  /* abort simulation */
    return(0);
  }
  if (vpi_scan(tfarg_itr) != NULL) {
    tf_error("ERROR: $list_pathout_ports requires 1 argument\n");
    vpi_free_object(tfarg_itr); /* because not scanning until null */
    vpi_control(vpiFinish, 1);  /* abort simulation */
    return(0);
  }
  return(0);
}

/**********************************************************************
 * calltf routine
 *********************************************************************/
PLI_INT32 PLIbook_PortInfo_calltf(PLI_BYTE8 *user_data)
{
  vpiHandle systf_h, arg_itr, mod_h,
            port_itr, port_h, lowconn_h, highconn_h;
  PLI_INT32 lowconn_type, highconn_type;

  /* get module handle from first system task argument.  Assume the  */
  /* compiletf routine has already verified correct argument type.   */
  systf_h = vpi_handle(vpiSysTfCall, NULL);
  if (systf_h == NULL) {
    vpi_printf("ERROR: list_pathout_ports could not obtain handle to systf call\n");
    return(0);
  }
  arg_itr = vpi_iterate(vpiArgument, systf_h);
  if (systf_h == NULL) {
    vpi_printf("ERROR: list_pathout_ports could not obtain iterator to systf args\n");
    return(0);
  }
  mod_h = vpi_scan(arg_itr);
  vpi_free_object(arg_itr); /* free itr since did not scan until nul */

  vpi_printf("\nModule %s (instance %s)\n",
             vpi_get_str(vpiDefName, mod_h),
             vpi_get_str(vpiFullName, mod_h));

  port_itr = vpi_iterate(vpiPort, mod_h);
  if (!port_itr) {
    vpi_printf("   No ports found\n");
    return(0);
  }
  while (port_h = vpi_scan(port_itr)) {
    vpi_printf("  Port name is %s\n", vpi_get_str(vpiName, port_h));
    vpi_printf("    Size is %d\n", vpi_get(vpiSize, port_h));
    switch (vpi_get(vpiDirection, port_h)) {
      case vpiInput:  vpi_printf("    Direction is input\n");  break;
      case vpiOutput: vpi_printf("    Direction is output\n"); break;
      case vpiInout:  vpi_printf("    Direction is inout\n");  break;
    }

    lowconn_h = vpi_handle(vpiLowConn, port_h);
    lowconn_type = vpi_get(vpiType, lowconn_h);
    vpi_printf("    Low conn data type is %s\n",
         vpi_get_str(vpiType, lowconn_h)); /* Verilog-2001 property */

    highconn_h = vpi_handle(vpiHighConn, port_h);
    if (!highconn_h) {
      vpi_printf("    No high conn\n");
      return(0);
    }
    highconn_type = vpi_get(vpiType, highconn_h);
    vpi_printf("    High conn data type is %s\n",
         vpi_get_str(vpiType, highconn_h)); /* Verilog-2001 property */
  }
  return(0);
}
/*********************************************************************/
