/**********************************************************************
 * $get_arg_val_test example -- PLI application using VPI routines
 *
 * C code to test the VPI "PLIbook_get_arg_val_vpi() application.
 *
 * Usage: initial
 *          $get_arg_val_test(arg1, arg2, arg3, arg4);
 *
 *  Where: arg1 is a reg or integer variable
 *         arg2 is a real number or variable
 *         arg3 is a literal string
 *         arg4 is a module instance name
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *********************************************************************/

#include <stdlib.h>    /* ANSI C standard library */
#include <stdio.h>     /* ANSI C standard input/output library */
#include <stdarg.h>    /* ANSI C standard arguments library */
#include "vpi_user.h"  /* IEEE 1364 PLI VPI routine library  */

/* prototypes of the PLI application routines */
PLI_INT32 PLIbook_GetArgValTestCall(PLI_BYTE8 *user_data);

#include "get_arg_val_vpi.c"  /* include the VPI applications */

/**********************************************************************
 * VPI Registration Data
 *********************************************************************/
void PLIbook_GetArgValTest_register()
{
  s_vpi_systf_data tf_data;
  tf_data.type        = vpiSysTask;
  tf_data.sysfunctype = 0;
  tf_data.tfname      = "$get_arg_val_test";
  tf_data.calltf      = PLIbook_GetArgValTestCall;
  tf_data.compiletf   = NULL;
  tf_data.sizetf      = NULL;
  tf_data.user_data   = NULL;
  vpi_register_systf(&tf_data);
}
/**********************************************************************/

/**********************************************************************
 * calltf routine to exercise various utility functions.
 *********************************************************************/
PLI_INT32 PLIbook_GetArgValTestCall(PLI_BYTE8 *user_data)
{
  int        int1;
  double     real1;
  PLI_BYTE8 *string1;
  vpiHandle  arg0_h, arg1_h, arg2_h, arg3_h, arg4_h, arg5_h, arg6_h;

  vpi_printf("\n*** Testing Utility Apps With Good Values ***\n");

  /* test PLIbook_get_arg_int_val_vpi() */
  vpi_printf("\n  Testing PLIbook_get_arg_int_val_vpi()...\n");
  int1 = PLIbook_get_arg_int_val_vpi(1);
  vpi_printf("    Value of arg 1 is %d:  EXPECTED 1\n", int1);

  /* test PLIbook_get_arg_real_val_vpi() */
  vpi_printf("\n  Testing PLIbook_get_arg_real_val_vpi()...\n");
  real1 = PLIbook_get_arg_real_val_vpi(2);
  vpi_printf("    Value of arg 2 is %1.1f:  EXPECTED 0.5\n", real1);

  /* test PLIbook_get_arg_string_val_vpi() */
  vpi_printf("\n  Testing PLIbook_get_arg_string_val_vpi()...\n");
  string1 = PLIbook_get_arg_string_val_vpi(3);
  vpi_printf("    Value of arg 3 is %s:  EXPECTED Hello world\n", string1);

  /* test PLIbook_get_arg_int_val_vpi() */
  vpi_printf("\n  Testing PLIbook_get_arg_int_val_vpi() to read a real value...\n");
  int1 = PLIbook_get_arg_int_val_vpi(2);
  vpi_printf("    Value of arg 2 is %d:  EXPECTED 1 (0.5 rounded)\n", int1);

  /* test PLIbook_get_arg_string_val_vpi() */
  vpi_printf("\n  Testing PLIbook_get_arg_string_val_vpi() on a real number...\n");
  string1 = PLIbook_get_arg_string_val_vpi(2);
  vpi_printf("    Value of arg 2 is %s:  EXPECTED 0.5\n", string1);


  vpi_printf("\n*** Testing Utility Routines With Bad Values ***\n");

  /* test PLIbook_get_arg_real_val_vpi() */
  vpi_printf("\n  Testing PLIbook_get_arg_int_val_vpi() on a module instance...\n");
  int1 = PLIbook_get_arg_int_val_vpi(4);
  vpi_printf("    Value of arg 4 is %d:  EXPECTED 0\n", int1);

  /* test PLIbook_get_arg_real_val_vpi() */
  vpi_printf("\n  Testing PLIbook_get_arg_real_val_vpi() on a module instance...\n");
  real1 = PLIbook_get_arg_real_val_vpi(4);
  vpi_printf("    Value of arg 4 is %1.1f:  EXPECTED 0.0\n", real1);

  /* test PLIbook_get_arg_real_val_vpi() */
  vpi_printf("\n  Testing PLIbook_get_arg_str_val_vpi() on a module instance...\n");
  string1 = PLIbook_get_arg_string_val_vpi(4);
  vpi_printf("    Value of arg 4 is %s:  EXPECTED <null string>\n", string1);

  /* test PLIbook_get_arg_int_val_vpi() */
  vpi_printf("\n  Testing PLIbook_get_arg_int_val_vpi() to read a literal string...\n");
  int1 = PLIbook_get_arg_int_val_vpi(3);
  vpi_printf("    Value of arg 3 is %x (hex):  EXPECTED 0 (hex)\n", int1);

  vpi_printf("\n*** All Tests Completed ***\n\n");
  return(0);
}
/*********************************************************************/
