/**********************************************************************
 * $set_mipd_delays example -- PLI application using VPI routines
 *
 * C source to write Module Input Port Delays onto input ports.
 *
 * Usage:
 *  $mipd_delays(module_input_port, rise_delay, fall_delay, toZ_delay);
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *********************************************************************/

#define VPI_1995 0 /* set to non-zero for Verilog-1995 compatibility */

#include <stdlib.h>    /* ANSI C standard library */
#include <stdio.h>     /* ANSI C standard input/output library */
#include <stdarg.h>    /* ANSI C standard arguments library */
#include "vpi_user.h"  /* IEEE 1364 PLI VPI routine library  */

#if VPI_1995
#include "../vpi_1995_compat.h"  /* kludge new Verilog-2001 routines */
#endif


/* prototypes of routines in this PLI application */
PLI_INT32  PLIbook_SetMipd_calltf(PLI_BYTE8 *user_data);
PLI_INT32  PLIbook_SetMipd_compiletf(PLI_BYTE8 *user_data);

/**********************************************************************
 * VPI Registration Data
 *********************************************************************/
void PLIbook_SetMipd_register()
{
  s_vpi_systf_data tf_data;   /* allocate register data structure */
  tf_data.type        = vpiSysTask;
  tf_data.sysfunctype = 0;
  tf_data.tfname      = "$set_mipd_delays";
  tf_data.calltf      = PLIbook_SetMipd_calltf;
  tf_data.compiletf   = PLIbook_SetMipd_compiletf;
  tf_data.sizetf      = NULL;
  tf_data.user_data   = NULL;
  vpi_register_systf(&tf_data);
}
/*********************************************************************/

/**********************************************************************
 * compiletf routine
 *********************************************************************/
PLI_INT32 PLIbook_SetMipd_compiletf(PLI_BYTE8 *user_data)
{
  vpiHandle   systf_h, tfarg_itr, tfarg_h, port_itr, port_h;
  int i;

  systf_h = vpi_handle(vpiSysTfCall, NULL);
  tfarg_itr = vpi_iterate(vpiArgument, systf_h);
  if (tfarg_itr == NULL) {
    vpi_printf("ERROR: $mipd_delays requires 4 arguments\n");
    vpi_control(vpiFinish, 1);  /* abort simulation */
    return(0);
  }

  tfarg_h = vpi_scan(tfarg_itr);
  if (vpi_get(vpiType, tfarg_h) != vpiNet) {
    vpi_printf("ERROR: $mipd_delays arg1 must be module input\n");
    vpi_control(vpiFinish, 1);  /* abort simulation */
    return(0);
  }

  port_itr = vpi_iterate(vpiPorts, tfarg_h);
  if (port_itr == NULL) {
    vpi_printf("$mipd_delays arg1 not connected to module port\n");
    vpi_control(vpiFinish, 1);  /* abort simulation */
    return(0);
  }
  port_h = vpi_scan(port_itr);
  vpi_free_object(port_itr); /* free iterator--did not scan to null */
  if (vpi_get(vpiType, port_h) != vpiPort) {
    vpi_printf("$mipd_delays arg1 not connected to an input port\n");
    vpi_control(vpiFinish, 1);  /* abort simulation */
    return(0);
  }

  for (i=2; i<=4; i++) {
    tfarg_h = vpi_scan(tfarg_itr);
    if (vpi_get(vpiType, tfarg_h) != vpiConstant) {
      vpi_printf("$mipd_delays arg %d must be a number\n", i);
      vpi_control(vpiFinish, 1);  /* abort simulation */
      return(0);
    }
  }
  if (vpi_scan(tfarg_itr) != NULL) {
    vpi_printf("ERROR: $mipd_delays requires only 4 arguments\n");
    vpi_free_object(tfarg_itr); /* free iterator--did not scan to null*/
    vpi_control(vpiFinish, 1);  /* abort simulation */
    return(0);
  }
  return(0);  /* no syntax errors detected */
}

/**********************************************************************
 * calltf routine
 *********************************************************************/
PLI_INT32 PLIbook_SetMipd_calltf(PLI_BYTE8 *user_data) {
  vpiHandle tfarg_itr, tfarg_h, port_itr, port_h;
  int i;
  s_vpi_delay delay_struct;    /* structure to delay setup */
  s_vpi_time  delay_array[3];  /* structure to hold delays */
  s_vpi_value tfarg_value;     /* structure to hold tfarg values */

  delay_struct.da           = delay_array;
  delay_struct.no_of_delays = 3;
  delay_struct.time_type    = vpiScaledRealTime;
  delay_struct.mtm_flag     = 0;
  delay_struct.append_flag  = 0;
  delay_struct.pulsere_flag = 0;
  tfarg_value.format        = vpiRealVal;

  /* obtain handle to system task arguments;
     compiletf has already verified only 4 args with correct types */
  tfarg_itr = vpi_iterate(vpiArgument, vpi_handle(vpiSysTfCall, NULL));
  tfarg_h = vpi_scan(tfarg_itr); /* read 1st tfarg */
  port_itr = vpi_iterate(vpiPorts, tfarg_h);
  port_h = vpi_scan(port_itr);
  if (port_h != NULL)
    vpi_free_object(port_itr); /* free iterator-did not scan to null */
  for (i=0; i<=2; i++) {
    vpi_get_value(vpi_scan(tfarg_itr), &tfarg_value); /* read tfargs */
    delay_array[i].real = tfarg_value.value.real;
  }
  vpi_free_object(tfarg_itr); /* free iterator--did not scan to null */

  vpi_put_delays(port_h, &delay_struct);

  return(0);
}

/*********************************************************************/

