/**********************************************************************
 * $scientific_alu example -- Verilog HDL test bench.
 *
 * Verilog test bench to test the $scientific_alu C model PLI
 * application.  This test uses the Scientific Alu as a sequential
 * logic device.
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *********************************************************************/

`include "sci_alu_sequential_shell.v"

`timescale 1ns / 1ns
module test;

  reg   [3:0] opcode;
  reg         clk;
  wire        excep, err;
  real        a, b, result;
  wire [63:0] a_in, b_in, result_out;
  integer     i;

  // convert real numbers to/from 64-bit vector port connections
  always @(result_out) result = $bitstoreal(result_out);
  assign a_in = $realtobits(a);
  assign b_in = $realtobits(b);

  scientific_alu i1 (clk, a_in, b_in, opcode, result_out, excep, err);

  always #5 clk = ~clk;

  initial
    begin
      $display("");  //add some white space to output
      $timeformat(-9,0," ns",7);
      opcode = 0;
      a = 16.0;
      b =  2.0;
      #0 clk = 0;
      for (i=1; i<=15; i=i+1)
        @(negedge clk) opcode = i;
      #10 $display("");
     $finish;
    end

  always @(clk)
    $strobe("At %t: clk=%b   result=%.2f \t excep=%b  err=%b  a=%.1f  b=%.1f  opcode=%h",
            $time, clk, result, excep, err, a, b, opcode);

endmodule

/*********************************************************************/
