/**********************************************************************
 * $strgetp_test example -- C source code using TF/ACC PLI routines
 *
 * C source to illustrate reading 4-state logic values into C strings.
 * The signals in Verilog to be read can be a vector of any bit size
 * (including scalar) or a real number, and can be negative or positive.
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *
 * Usage:
 * ------
 *
 *   Syntax:   $strgetp_test(<signal_or_literal_value>);
 *
 *   Example:
 *    reg [0:39] data;
 *    initial $strgetp_test(data);
 *
 * Routine definitions for a veriusertfs array:
 * /* routine prototypes -/
 * extern int PLIbook_strgetpTest_checktf(int user_data, int reason);
 * extern int PLIbook_strgetpTest_calltf(int user_data, int reason);
 * /* table entries -/
 *  {usertask,                         /* type of PLI routine -/
 *    0,                               /* user_data value -/
 *    PLIbook_strgetpTest_checktf,     /* checktf routine -/
 *    0,                               /* sizetf routine -/
 *    PLIbook_strgetpTest_calltf,      /* calltf routine -/
 *    0,                               /* misctf routine -/
 *    "$strgetp_test",                 /* system task/function name -/
 *    1                                /* forward reference = true -/
 *  },
 *********************************************************************/

#include <stdlib.h>    /* ANSI C standard library */
#include <stdio.h>     /* ANSI C standard input/output library */
#include "veriuser.h"  /* IEEE 1364 PLI TF routine library  */

/**********************************************************************
 * checktf routine
 *********************************************************************/
int PLIbook_strgetpTest_checktf(int user_data, int reason)
{
  if (tf_nump() != 1)
    tf_error("Usage error: $strgetp_test(<signal>);");
  return(0);
}

/**********************************************************************
 * calltf routine
 *********************************************************************/
int PLIbook_strgetpTest_calltf(int user_data, int reason)
{
  PLI_BYTE8 *dec, *bin, *hex;

  dec = tf_strgetp(1, 'd');
  bin = tf_strgetp(1, 'b');
  hex = tf_strgetp(1, 'h');
  if (dec != null) {
    io_printf("$strgetp_test arg value is %s (decimal)\n", dec);
    io_printf("$strgetp_test arg value is %s (binary)\n", bin);
    io_printf("$strgetp_test arg value is %s (hex)\n", hex);
  }
  else
    io_printf("ERROR: $strgetp_test could not read arg value\n");

  return(0);
}

/*********************************************************************/
