/**********************************************************************
 * Example veriusertfs table that is used by many Verilog simulators
 * to register PLI applications that use the TF and ACC libraries of
 * the IEEE 1364 PLI.
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *********************************************************************/

/* prototypes of the PLI application routines */
extern int PLIbook_realpow_checktf(int user_data, int reason);
extern int PLIbook_realpow_calltf(int user_data, int reason);

extern int PLIbook_strgetpTest_checktf(int user_data, int reason);
extern int PLIbook_strgetpTest_calltf(int user_data, int reason);

extern int PLIbook_exprinfoTest_checktf(int user_data, int reason);
extern int PLIbook_exprinfoTest_calltf(int user_data, int reason);

extern int PLIbook_evaluatepTest_checktf(int user_data, int reason);
extern int PLIbook_evaluatepTest_misctf(int user_data, int reason, int pvc);
extern int PLIbook_evaluatepTest_calltf(int user_data, int reason);

extern int PLIbook_propagatepTest_checktf(int user_data, int reason);
extern int PLIbook_propagatepTest_calltf(int user_data, int reason);
extern int PLIbook_propagatepTest_misctf(int user_data, int reason, int pvc);

extern int PLIbook_nodeinfoTest_checktf(int user_data, int reason);
extern int PLIbook_nodeinfoTest_calltf(int user_data, int reason);

extern int PLIbook_DumpMem_checktf(int user_data, int reason);
extern int PLIbook_DumpMem_calltf(int user_data, int reason);
extern int PLIbook_DumpMem_misctf(int user_data, int reason, int pvc);

extern int PLIbook_FillMem_checktf(int user_data, int reason);
extern int PLIbook_FillMem_calltf(int user_data, int reason);
extern int PLIbook_FillMem_misctf(int user_data, int reason, int pvc);

/* the veriusertfs table */
s_tfcell veriusertfs[] =
{
  {userrealfunction,                 /* type of PLI routine */
    0,                               /* user_data value */
    PLIbook_realpow_checktf,         /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_realpow_calltf,          /* calltf routine */
    0,                               /* misctf routine */
    "$realpow",                      /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {usertask,                         /* type of PLI routine */
    0,                               /* user_data value */
    PLIbook_strgetpTest_checktf,     /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_strgetpTest_calltf,      /* calltf routine */
    0,                               /* misctf routine */
    "$strgetp_test",                 /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {usertask,                         /* type of PLI routine */
    0,                               /* user_data value */
    PLIbook_exprinfoTest_checktf,    /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_exprinfoTest_calltf,     /* calltf routine */
    0,                               /* misctf routine */
    "$exprinfo_test",                /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {usertask,                         /* type of PLI routine */
    0,                               /* user_data value */
    PLIbook_evaluatepTest_checktf,   /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_evaluatepTest_calltf,    /* calltf routine */
    PLIbook_evaluatepTest_misctf,    /* misctf routine */
    "$evaluatep_test",               /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {usertask,                         /* type of PLI routine */
    0,                               /* user_data value */
    PLIbook_propagatepTest_checktf,  /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_propagatepTest_calltf,   /* calltf routine */
    PLIbook_propagatepTest_misctf,   /* misctf routine */
    "$propagatep_test",              /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {usertask,                         /* type of PLI routine */
    0,                               /* user_data value */
    PLIbook_nodeinfoTest_checktf,    /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_nodeinfoTest_calltf,     /* calltf routine */
    0,                               /* misctf routine */
    "$nodeinfo_test",                /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {usertask,                         /* type of PLI routine */
    0,                               /* user_data value */
    PLIbook_DumpMem_checktf,         /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_DumpMem_calltf,          /* calltf routine */
    PLIbook_DumpMem_misctf,          /* misctf routine */
    "$dump_mem_hex",                 /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {usertask,                         /* type of PLI routine */
    1,                               /* user_data value */
    PLIbook_DumpMem_checktf,         /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_DumpMem_calltf,          /* calltf routine */
    PLIbook_DumpMem_misctf,          /* misctf routine */
    "$dump_mem_bin",                 /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {usertask,                         /* type of PLI routine */
    2,                               /* user_data value */
    PLIbook_DumpMem_checktf,         /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_DumpMem_calltf,          /* calltf routine */
    PLIbook_DumpMem_misctf,          /* misctf routine */
    "$dump_mem_ascii",               /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {usertask,                         /* type of PLI routine */
    0,                               /* user_data value */
    PLIbook_FillMem_checktf,         /* checktf routine */
    0,                               /* sizetf routine */
    PLIbook_FillMem_calltf,          /* calltf routine */
    PLIbook_FillMem_misctf,          /* misctf routine */
    "$fill_mem",                     /* system task/function name */
    1                                /* forward reference = true */
  },                                 /* */

  {0} /*** final entry must be 0 ***/
};

