/**********************************************************************
 * $change_path_delays example -- C source code using ACC PLI routines
 *
 * C source to scale the pin-to-pin path delays of a module. The module
 * instance and scale factor are passed as system task arguments.
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *
 * Usage: $change_path_delays(<port_name>, <scale_factor>);
 *
 * Routine definitions for a veriusertfs array:
 * /* routine prototypes -/
 * extern int PLIbook_AddDelays_checktf(int user_data, int reason);
 * extern int PLIbook_AddDelays_calltf(int user_data, int reason);
 * /* table entries -/
 * {usertask,                       /* type of PLI routine -/
 *   0,                             /* user_data value -/
 *   PLIbook_ChangeDelays_checktf,  /* checktf routine -/
 *   0,                             /* sizetf routine -/
 *   PLIbook_ChangeDelays_calltf,   /* calltf routine -/
 *   0,                             /* misctf routine -/
 *   "$change_path_delays",         /* system task/function name -/
 *   1                              /* forward reference = true -/
 * },
 *********************************************************************/

#include <stdlib.h>    /* ANSI C standard library */
#include <stdio.h>     /* ANSI C standard input/output library */
#include "veriuser.h"  /* IEEE 1364 PLI TF routine library  */
#include "acc_user.h"  /* IEEE 1364 PLI ACC routine library */
/**********************************************************************
 * checktf routine
 *********************************************************************/
int PLIbook_ChangeDelays_checktf(int user_data, int reason)
{
  acc_initialize();
  if (tf_nump() != 2)
    tf_error("$list_path_delays must have 2 arguments.");
  else if (tf_typep(1) == TF_NULLPARAM)
    tf_error("$list_path_delays arg cannot be null.");
  else if (acc_fetch_type(acc_handle_tfarg(1)) != accModule)
    tf_error("$list_path_delays arg must be a module instance.");
  acc_close();
  return(0);
}

/**********************************************************************
 * calltf routine
 *********************************************************************/
int PLIbook_ChangeDelays_calltf(int user_data, int reason)
{
  double rise, fall, toZ;
  double scale_factor;
  handle module_h, path_h;
  int path_count = 0;

  acc_initialize();
  acc_configure(accMinTypMaxDelays, "false");
  acc_configure(accPathDelayCount,  "3");

  module_h = acc_handle_tfarg(1);
  io_printf("\nIn module: %s\n", acc_fetch_fullname(module_h));

  scale_factor = acc_fetch_tfarg(2);

  path_h = null;    /* start with known value for target handle */
  while (path_h = acc_next_modpath(module_h, path_h)) {
    path_count++;
    acc_fetch_delays(path_h, &rise, &fall, &toZ);
    io_printf("  Path %s:\n", acc_fetch_name(path_h));
    io_printf("    Original delay values are (%2.2f,%2.2f,%2.2f):\n",
              rise, fall, toZ);
    rise *= scale_factor;
    fall *= scale_factor;
    toZ  *= scale_factor;

//    acc_replace_delays(path_h, rise, fall, toZ);

    acc_fetch_delays(path_h, &rise, &fall, &toZ);
    io_printf("    After scaling, values are (%2.2f,%2.2f,%2.2f):\n",
              rise, fall, toZ);
  io_printf("\n");
  }
  if (!path_count) {
    io_printf("  no module paths found\n");
  }
  acc_close();
  return(0);
}
/*********************************************************************/
