/**********************************************************************
 * $change_path_delays example -- Verilog test bench source code
 *
 * Verilog test bench to test the $change_path_delays PLI application.
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *********************************************************************/
`timescale 1ns / 100ps
module top;
  wire [3:0] in1, in2;
  wire       carry_in;
  wire [3:0] result;
  wire       carry_out;

  add4 u1 (in1, in2, carry_in, result, carry_out);

  initial
    begin
      $change_path_delays(top.u1, 1.7);
      #1 $finish;
    end
endmodule

/*** A 4-bit adder model, made of 1-bit adders ***/
`timescale 1ns / 10ps
module add4 (r1, r2, ci, out, co);
 input   [3:0] r1, r2;
 input         ci;
 output  [3:0] out;
 output        co;
 wire    [3:0] r1, r2, out;
 wire          ci, c1, c2, c3, co;

 addbit i1 (r1[0],r2[0],ci,out[0],c1);
 addbit i2 (r1[1],r2[1],c1,out[1],c2);
 addbit i3 (r1[2],r2[2],c2,out[2],c3);
 addbit i4 (r1[3],r2[3],c3,out[3],co);

 specify
   (r1 *> out) = (1.1, 1.2);
   (r2 *> out) = (1.5, 1.6);
   (ci *> out) = (1.8, 1.9);
 endspecify
endmodule

/*** A gate level 1 bit adder model ***/
`celldefine
`timescale 1ns / 1ns
module addbit (a, b, ci, sum, co);
  input  a, b, ci;
  output sum, co;

  wire  a, b, ci, sum, co, n1, n2, n3, n4;

  xor p1 (n1, a, b);
  xor p2 (sum, n1, ci);
  and p3 (n2, a, b);
  and p4 (n3, n1, ci);
  or  p5 (co, n2, n3);
endmodule
`endcelldefine
/*********************************************************************/

