/**********************************************************************
 * $mipd_delays example -- C source code using ACC PLI routines
 *
 * C source to modify the delays of a verilog gate primitive. the
 * gate instance and delay values to be added to the existing gate
 * delays are passed as system task arguments.
 *
 * For the book, "The Verilog PLI Handbook" by Stuart Sutherland
 *  Copyright 1999 & 2001, Kluwer Academic Publishers, Norwell, MA, USA
 *   Contact: www.wkap.il
 *  Example copyright 1998, Sutherland HDL Inc, Portland, Oregon, USA
 *   Contact: www.sutherland-hdl.com
 *
 * Usage: $mipd_delays(<port_name>, <d1>, <d2>, ... <d9>);
 *
 * Routine definitions for a veriusertfs array:
 * /* routine prototypes -/
 * extern int PLIbook_AddDelays_checktf(int user_data, int reason);
 * extern int PLIbook_AddDelays_calltf(int user_data, int reason);
 * /* table entries -/
 * {usertask,                       /* type of PLI routine -/
 *   0,                             /* user_data value -/
 *   0,                             /* checktf routine -/
 *   0,                             /* sizetf routine -/
 *   PLIbook_MipdDelays_calltf,     /* calltf routine -/
 *   0,                             /* misctf routine -/
 *   "$mipd_delays",                /* system task/function name -/
 *   1                              /* forward reference = true -/
 * },
 *********************************************************************/

#include <stdlib.h>    /* ANSI C standard library */
#include <stdio.h>     /* ANSI C standard input/output library */
#include "veriuser.h"  /* IEEE 1364 PLI TF routine library  */
#include "acc_user.h"  /* IEEE 1364 PLI ACC routine library */

#define DEBUG 0        /* set to non-zero for debug messages */

/**********************************************************************
 * calltf routine
 *********************************************************************/
int PLIbook_MipdDelays_calltf(int user_data, int reason)
{
  double delay_array[9];
  double rise, fall, toZ;
  handle port_h;
  int i;
  PLI_INT32 arg_type;

  acc_initialize();
  acc_configure(accDisplayWarnings, "true");

  acc_configure(accMinTypMaxDelays, "true");
  port_h = acc_handle_tfarg(1);

  arg_type = acc_fetch_type(port_h);

  #if DEBUG
    io_printf("\nDEBUG: acc_handle_tfarg(1) object type is %s / %s\n",
              acc_fetch_type_str(arg_type),
              acc_fetch_type_str(arg_type));
  #endif

  /* most simulators return loconn handle, not port handle */
  switch (arg_type) {
    case accPort:
    case accScalarPort: break;  /* arg type is the right type */
    case accPortBit: port_h = acc_handle_parent(port_h); break;
    case accNet:
    case accNetBit: port_h = acc_next_port(port_h, null); break;
    default: io_printf("\nERROR in $mipd_delays: acc_handle_tfarg(1) returned illegal type of %s\n\n", acc_fetch_type_str(arg_type)); return(0);
  }

  /* see if now have a port handle */
  arg_type = acc_fetch_type(port_h);
  switch (arg_type) {
    case accPort:
    case accScalarPort: break;  /* arg type is the right type */
    case accPortBit: port_h = acc_handle_parent(port_h); break;
    default: io_printf("\nERROR in $mipd_delays: could not obtain port handle; ended up with type of %s\n\n", acc_fetch_type_str(arg_type)); return(0);
  }

  #if DEBUG
    arg_type = acc_fetch_type(port_h);
    io_printf("DEBUG: after type checks, port_h type is %s / %s\n",
              acc_fetch_type_str(arg_type),
              acc_fetch_type_str(arg_type));
    io_printf("DEBUG: the port handle is to %s\n\n",
              acc_fetch_fullname(port_h));
  #endif

  for (i = 0; i <= 8; i++) {
    delay_array[i] = acc_fetch_tfarg(i+2);
  }

  #if DEBUG
    for (i = 0; i <= 8; i++) {
    io_printf("DEBUG: delay_array[%d] has value %f\n",
              i, delay_array[i]);
    io_printf("\n");
    }
  #endif

  acc_replace_delays(port_h, delay_array);

  /* verify new delays took affect */
  acc_configure(accMinTypMaxDelays, "false");
  rise = fall = toZ = 0.0;
  acc_fetch_delays(port_h, &rise, &fall, &toZ);
  io_printf("Port %s new delays: (%1.2f, %1.2f, %1.2f)\n\n",
              acc_fetch_name(port_h),
              rise, fall, toZ);
  acc_close();
  return(0);
}
/*********************************************************************/
